==> torch_npu-2.1.0-cp38-cp38-manylinux_2_17_aarch64.manylinux2014_aarch64.whl: torch_npu/profiler/analysis/prof_parse/cann_file_parser.py <==
import ast
import json
import os
import re
import subprocess
import shutil
from enum import Enum
from json import JSONDecodeError

from ....utils.path_manager import PathManager
from ..prof_bean.event_bean import EventBean
from ..prof_common_func.constant import Constant, print_warn_msg
from ..prof_common_func.constant import print_error_msg
from ..prof_common_func.file_manager import FileManager
from ..prof_common_func.path_manager import ProfilerPathManager
from ..prof_bean.step_trace_bean import StepTraceBean


class CANNDataEnum(Enum):
    OP_SUMMARY = 0
    NPU_MEMORY = 1
    MSPROF_TIMELINE = 2
    STEP_TRACE = 3
    GE_MEMORY_RECORD = 4
    GE_OPERATOR_MEMORY = 5
    L2_CACHE = 6
    AI_CPU = 7
    COMMUNICATION = 8
    MATRIX = 9
    NPU_MODULE_MEM = 11


class CANNFileParser:
    COMMAND_SUCCESS = 0
    ACL_TO_NPU = "acl_to_npu"
    START_FLOW = "s"
    END_FLOW = "f"
    SUMMARY = "summary"
    TIMELINE = "timeline"
    ANALYZE = "analyze"
    CANN_DATA_MATCH = {
        CANNDataEnum.OP_SUMMARY: [r"^op_summary_\d+_\d+\.csv", r"^op_summary_\d+_\d+_\d+\.csv",
                                  r"^op_summary_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.NPU_MEMORY: [r"^npu_mem_\d+_\d+\.csv", r"^npu_mem_\d+_\d+_\d+\.csv",
                                  r"^npu_mem_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.MSPROF_TIMELINE: [r"^msprof_\d+_\d+\.json", r"^msprof_\d+_\d+_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_\d+\.json", r"^msprof_\d+_\d+_slice_\d+\.json",
                                       r"^msprof_\d+_\d+_slice_\d+_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_slice_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_slice_\d+_\d+\.json"],
        CANNDataEnum.STEP_TRACE: [r"^step_trace_\d+_\d+\.csv", r"^step_trace_\d+_\d+_\d+\.csv",
                                  r"^step_trace_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.GE_MEMORY_RECORD: [r"^ge_memory_record_\d+_\d+\.csv", r"^ge_memory_record_\d+_\d+_\d+\.csv",
                                        r"^ge_memory_record_\d+_\d+_\d+_\d+\.csv", r"^memory_record_\d+_\d+\.csv",
                                        r"^memory_record_\d+_\d+_\d+\.csv", r"^memory_record_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.GE_OPERATOR_MEMORY: [r"^ge_operator_memory_\d+_\d+\.csv", r"^ge_operator_memory_\d+_\d+_\d+\.csv",
                                          r"^ge_operator_memory_\d+_\d+_\d+_\d+\.csv", r"^operator_memory_\d+_\d+\.csv",
                                          r"^operator_memory_\d+_\d+_\d+\.csv",
                                          r"^operator_memory_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.L2_CACHE: [r"^l2_cache_\d+_\d+\.csv", r"^l2_cache_\d+_\d+_\d+\.csv",
                                r"^l2_cache_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.AI_CPU: [r"^aicpu_\d+_\d+\.csv", r"^aicpu_\d+_\d+_\d+\.csv", r"^aicpu_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.COMMUNICATION: [r"^communication\.json"],
        CANNDataEnum.MATRIX: [r"^communication_matrix\.json"],
        CANNDataEnum.NPU_MODULE_MEM: [r"^npu_module_mem_\d_\d_\d+\.csv"],
    }

    def __init__(self, profiler_path: str):
        self._cann_path = ProfilerPathManager.get_cann_path(profiler_path)
        self._file_dict = {}
        self._file_dispatch()
        self.msprof_path = shutil.which("msprof")

    @classmethod
    def _json_load(cls, data: str) -> list:
        if not data:
            return []
        try:
            data = json.loads(data)
        except JSONDecodeError as e:
            raise RuntimeError("Invalid CANN trace data.") from e
        if not isinstance(data, list):
            return []
        if data and not isinstance(data[0], dict):
            return []
        return data

    @classmethod
    def _json_dict_load(cls, data: str) -> dict:
        if not data:
            return {}
        try:
            data = json.loads(data)
        except JSONDecodeError as e:
            raise RuntimeError("Invalid communication data.") from e
        if not isinstance(data, dict):
            return {}
        return data

    @staticmethod
    def _get_data_simplification_cmd(data_simplification: bool):
        switch = "on" if data_simplification else "off"
        return f"--clear={switch}"

    def export_cann_profiling(self, data_simplification: bool):
        if not os.path.isdir(self._cann_path):
            return
        self._del_summary_and_timeline_data()
        if not self.msprof_path:
            err_msg = "Export CANN Profiling data failed! msprof command not found!"
            print_error_msg(err_msg)
            raise RuntimeError(err_msg)
        completed_process = subprocess.run([self.msprof_path, "--export=on", f"--output={self._cann_path}"],
                                           capture_output=True, shell=False)
        if completed_process.returncode != self.COMMAND_SUCCESS:
            raise RuntimeError(
                f"Export CANN Profiling data failed, please verify that the ascend-toolkit is installed and set-env.sh "
                f"is sourced. or you can execute the command to confirm the CANN Profiling export result: "
                f"msprof --export=on --output={self._cann_path}")

        self._file_dispatch()
        step_trace_file_set = self.get_file_list_by_type(CANNDataEnum.STEP_TRACE)
        if step_trace_file_set:
            step_file = step_trace_file_set.pop()
            step_trace_data = FileManager.read_csv_file(step_file, StepTraceBean)
            # step file name: step_trace_{device id}_{model id}_{iteration id}_{时间戳}.csv
            parsed_step = int(os.path.basename(step_file).split(".")[0].split("_")[-2])
            for data in step_trace_data:
                step_id = data.step_id
                if step_id != Constant.INVALID_VALUE and step_id != parsed_step:
                    completed_process = subprocess.run(
                        [self.msprof_path, "--export=on", f"--output={self._cann_path}", f"--iteration-id={step_id}"],
                        capture_output=True, shell=False)
                    if completed_process.returncode != self.COMMAND_SUCCESS:
                        raise RuntimeError("Export CANN Profiling data failed, please verify that the "
                                           "ascend-toolkit is installed and set-env.sh is sourced.")

        simplification_cmd = self._get_data_simplification_cmd(data_simplification)
        completed_analysis = subprocess.run(
            [self.msprof_path, "--analyze=on", f"--output={self._cann_path}", simplification_cmd],
            capture_output=True, shell=False)
        if completed_analysis.returncode != self.COMMAND_SUCCESS:
            print_warn_msg("Analyze CANN Profiling data failed.")

    def get_timeline_all_data(self) -> list:
        timeline_data = []
        msprof_file_list = self._file_dict.get(CANNDataEnum.MSPROF_TIMELINE, set())
        for msprof_file in msprof_file_list:
            data = self._json_load(FileManager.file_read_all(msprof_file, "rt"))
            timeline_data.extend(data)
        return timeline_data

    def get_analyze_communication_data(self, file_type: Enum) -> dict:
        communication_data = {}
        communication_file_set = self._file_dict.get(file_type, set())
        if communication_file_set:
            # only need to read one file if there exist more than one files
            sub_file = next(iter(communication_file_set))
            communication_data = self._json_dict_load(FileManager.file_read_all(sub_file, "rt"))
        return communication_data

    def get_acl_to_npu_data(self) -> dict:
        flow_start_dict, flow_end_dict = {}, {}
        all_data = self.get_timeline_all_data()
        for data in all_data:
            event_bean = EventBean(data)
            if event_bean.is_flow_start_event():
                flow_start_dict[event_bean.id] = event_bean.ts
            elif event_bean.is_flow_end_event():
                flow_end_dict[event_bean.unique_id] = event_bean.id
        acl_to_npu_dict = {}
        for data in all_data:
            event_bean = EventBean(data)
            if event_bean.is_x_event():
                corr_id = flow_end_dict.get(event_bean.unique_id)
                acl_ts = flow_start_dict.get(corr_id)
                if corr_id is not None and acl_ts is not None:
                    acl_to_npu_dict.setdefault(acl_ts, []).append(event_bean)
        return acl_to_npu_dict

    def get_file_list_by_type(self, file_type: CANNDataEnum) -> set:
        return self._file_dict.get(file_type, set())

    def check_prof_data_size(self):
        if not self._cann_path:
            return
        device_data_path = os.path.join(ProfilerPathManager.get_device_path(self._cann_path), "data")
        host_data_path = os.path.join(self._cann_path, "host", "data")
        prof_data_size = 0
        for root, dirs, files in os.walk(device_data_path):
            prof_data_size += sum([os.path.getsize(os.path.join(root, name)) for name in files])
        for root, dirs, files in os.walk(host_data_path):
            prof_data_size += sum([os.path.getsize(os.path.join(root, name)) for name in files])
        if prof_data_size >= Constant.PROF_WARN_SIZE:
            print_warn_msg("The parsing time is expected to exceed 30 minutes, "
                           "and you can choose to stop the process and use offline parsing.")

    def get_localtime_diff(self) -> float:
        localtime_diff = 0
        if not self._cann_path:
            return localtime_diff
        start_info_path = ProfilerPathManager.get_start_info_path(self._cann_path)
        if not start_info_path:
            return localtime_diff
        try:
            info_json = ast.literal_eval(FileManager.file_read_all(start_info_path, "rt"))
            localtime_diff = float(info_json.get(Constant.CANN_BEGIN_TIME, 0)) - float(
                info_json.get(Constant.CANN_BEGIN_MONOTONIC, 0)) / Constant.NS_TO_US
        except Exception:
            print_warn_msg("Failed to get CANN localtime diff.")
        return localtime_diff

    def _file_dispatch(self):
        all_file_list = ProfilerPathManager.get_device_all_file_list_by_type(self._cann_path, self.SUMMARY)
        all_file_list += ProfilerPathManager.get_device_all_file_list_by_type(self._cann_path, self.TIMELINE)
        all_file_list += ProfilerPathManager.get_analyze_all_file(self._cann_path, self.ANALYZE)
        for file_path in all_file_list:
            if not os.path.isfile(file_path):
                continue
            for data_type, re_match_exp_list in self.CANN_DATA_MATCH.items():
                for re_match_exp in re_match_exp_list:
                    if re.match(re_match_exp, os.path.basename(file_path)):
                        self._file_dict.setdefault(data_type, set()).add(file_path)

    def _del_summary_and_timeline_data(self):
        device_path = ProfilerPathManager.get_device_path(self._cann_path)
        if not device_path:
            return
        summary_path = os.path.join(device_path, "summary")
        timeline_path = os.path.join(device_path, "timeline")
        PathManager.remove_path_safety(summary_path)
        PathManager.remove_path_safety(timeline_path)

==> torch_npu-2.2.0rc1-cp38-cp38-manylinux_2_17_aarch64.manylinux2014_aarch64.whl: torch_npu/profiler/analysis/prof_parse/cann_file_parser.py <==
import ast
import json
import os
import re
from enum import Enum
from json import JSONDecodeError

from ....utils.path_manager import PathManager
from ..prof_bean.event_bean import EventBean
from ..prof_common_func.constant import Constant, print_warn_msg
from ..prof_common_func.constant import convert_us2ns
from ..prof_common_func.path_manager import ProfilerPathManager
from ..prof_common_func.file_manager import FileManager


class CANNDataEnum(Enum):
    OP_SUMMARY = 0
    NPU_MEMORY = 1
    MSPROF_TIMELINE = 2
    STEP_TRACE = 3
    GE_MEMORY_RECORD = 4
    GE_OPERATOR_MEMORY = 5
    L2_CACHE = 6
    AI_CPU = 7
    COMMUNICATION = 8
    MATRIX = 9
    OP_STATISTIC = 10
    NPU_MODULE_MEM = 11


class CANNFileParser:
    START_FLOW = "s"
    END_FLOW = "f"
    SUMMARY = "summary"
    TIMELINE = "timeline"
    ANALYZE = "analyze"
    HOST_TO_DEVICE = "HostToDevice"
    CANN_DATA_MATCH = {
        CANNDataEnum.OP_SUMMARY: [r"^op_summary_\d+_\d+\.csv", r"^op_summary_\d+_\d+_\d+\.csv",
                                  r"^op_summary_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.NPU_MEMORY: [r"^npu_mem_\d+_\d+\.csv", r"^npu_mem_\d+_\d+_\d+\.csv",
                                  r"^npu_mem_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.MSPROF_TIMELINE: [r"^msprof_\d+_\d+\.json", r"^msprof_\d+_\d+_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_\d+\.json", r"^msprof_\d+_\d+_slice_\d+\.json",
                                       r"^msprof_\d+_\d+_slice_\d+_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_slice_\d+\.json",
                                       r"^msprof_\d+_\d+_\d+_slice_\d+_\d+\.json"],
        CANNDataEnum.STEP_TRACE: [r"^step_trace_\d+_\d+\.csv", r"^step_trace_\d+_\d+_\d+\.csv",
                                  r"^step_trace_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.GE_MEMORY_RECORD: [r"^ge_memory_record_\d+_\d+\.csv", r"^ge_memory_record_\d+_\d+_\d+\.csv",
                                        r"^ge_memory_record_\d+_\d+_\d+_\d+\.csv", r"^memory_record_\d+_\d+\.csv",
                                        r"^memory_record_\d+_\d+_\d+\.csv", r"^memory_record_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.GE_OPERATOR_MEMORY: [r"^ge_operator_memory_\d+_\d+\.csv", r"^ge_operator_memory_\d+_\d+_\d+\.csv",
                                          r"^ge_operator_memory_\d+_\d+_\d+_\d+\.csv", r"^operator_memory_\d+_\d+\.csv",
                                          r"^operator_memory_\d+_\d+_\d+\.csv",
                                          r"^operator_memory_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.L2_CACHE: [r"^l2_cache_\d+_\d+\.csv", r"^l2_cache_\d+_\d+_\d+\.csv",
                                r"^l2_cache_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.AI_CPU: [r"^aicpu_\d+_\d+\.csv", r"^aicpu_\d+_\d+_\d+\.csv", r"^aicpu_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.COMMUNICATION: [r"^communication\.json"],
        CANNDataEnum.MATRIX: [r"^communication_matrix\.json"],
        CANNDataEnum.OP_STATISTIC: [r"^op_statistic_\d+_\d+\.csv", r"^op_statistic_\d+_\d+_\d+\.csv",
                                    r"^op_statistic_\d+_\d+_\d+_\d+\.csv"],
        CANNDataEnum.NPU_MODULE_MEM: [r"^npu_module_mem_\d_\d_\d+\.csv"],
    }

    def __init__(self, profiler_path: str):
        self._cann_path = ProfilerPathManager.get_cann_path(profiler_path)
        self._file_dict = {}
        self._file_dispatch()

    @classmethod
    def _json_load(cls, data: str) -> list:
        if not data:
            return []
        try:
            data = json.loads(data)
        except JSONDecodeError as e:
            raise RuntimeError("Invalid CANN trace data.") from e
        if not isinstance(data, list):
            return []
        if data and not isinstance(data[0], dict):
            return []
        return data

    @classmethod
    def _json_dict_load(cls, data: str) -> dict:
        if not data:
            return {}
        try:
            data = json.loads(data)
        except JSONDecodeError as e:
            raise RuntimeError("Invalid communication data.") from e
        if not isinstance(data, dict):
            return {}
        return data

    @classmethod
    def combine_acl_to_npu(cls, timeline_data: list) -> dict:
        flow_dict, event_dict = {}, {}
        for data in timeline_data:
            if data.get("cat") == cls.HOST_TO_DEVICE and data.get("ph") == cls.START_FLOW:
                flow_dict.setdefault(data.get("id", 0), {}).setdefault("start", data)
                continue
            if data.get("cat") == cls.HOST_TO_DEVICE and data.get("ph") == cls.END_FLOW:
                flow_dict.setdefault(data.get("id", 0), {}).setdefault("end", data)
                continue
            if data.get("ph") == "X":
                pid = data.get("pid")
                tid = data.get("tid")
                ts = data.get("ts")
                unique_id = f"{pid}-{tid}-{ts}"
                event_dict[unique_id] = data
        acl_to_npu_dict = {}
        for flow in flow_dict.values():
            start_event = flow.get("start")
            end_event = flow.get("end")
            if start_event and end_event:
                pid = end_event.get("pid")
                tid = end_event.get("tid")
                ts = end_event.get("ts")
                unique_id = f"{pid}-{tid}-{ts}"
                kernel_event = event_dict.get(unique_id)
                if not kernel_event:
                    continue
                acl_to_npu_dict.setdefault(convert_us2ns(start_event.get("ts", 0)), []).append(EventBean(kernel_event))
        return acl_to_npu_dict

    def get_timeline_all_data(self) -> list:
        timeline_data = []
        msprof_file_list = self._file_dict.get(CANNDataEnum.MSPROF_TIMELINE, set())
        for msprof_file in msprof_file_list:
            data = self._json_load(FileManager.file_read_all(msprof_file, "rt"))
            timeline_data.extend(data)
        return timeline_data

    def get_analyze_communication_data(self, file_type: Enum) -> dict:
        communication_data = {}
        communication_file_set = self._file_dict.get(file_type, set())
        if communication_file_set:
            # only need to read one file if there exist more than one files
            sub_file = next(iter(communication_file_set))
            communication_data = self._json_dict_load(FileManager.file_read_all(sub_file, "rt"))
        return communication_data

    def get_acl_to_npu_data(self):
        return self.combine_acl_to_npu(self.get_timeline_all_data())

    def get_file_list_by_type(self, file_type: CANNDataEnum) -> set:
        return self._file_dict.get(file_type, set())

    def get_localtime_diff(self) -> float:
        localtime_diff = 0
        if not self._cann_path:
            return localtime_diff
        start_info_path = ProfilerPathManager.get_start_info_path(self._cann_path)
        if not start_info_path:
            return localtime_diff
        try:
            info_json = ast.literal_eval(FileManager.file_read_all(start_info_path, "rt"))
            localtime_diff = convert_us2ns(info_json.get(Constant.CANN_BEGIN_TIME, 0)) - int(
                info_json.get(Constant.CANN_BEGIN_MONOTONIC, 0))
        except Exception:
            print_warn_msg("Failed to get CANN localtime diff.")
        return localtime_diff

    def del_summary_and_timeline_data(self):
        device_path = ProfilerPathManager.get_device_path(self._cann_path)
        if not device_path:
            return
        summary_path = os.path.join(device_path, "summary")
        timeline_path = os.path.join(device_path, "timeline")
        PathManager.remove_path_safety(summary_path)
        PathManager.remove_path_safety(timeline_path)

    def _file_dispatch(self):
        all_file_list = ProfilerPathManager.get_device_all_file_list_by_type(self._cann_path, self.SUMMARY)
        all_file_list += ProfilerPathManager.get_device_all_file_list_by_type(self._cann_path, self.TIMELINE)
        all_file_list += ProfilerPathManager.get_analyze_all_file(self._cann_path, self.ANALYZE)
        for file_path in all_file_list:
            if not os.path.isfile(file_path):
                continue
            for data_type, re_match_exp_list in self.CANN_DATA_MATCH.items():
                for re_match_exp in re_match_exp_list:
                    if re.match(re_match_exp, os.path.basename(file_path)):
                        self._file_dict.setdefault(data_type, set()).add(file_path)
